'''
Import builtin functions for easy recovery when a script or a preparation
routine messes with Python's __builtins__.
'''

# Save this for later
__name = __name__
__package = __package__

# Make source code analysis happy
print = print  # @ReservedAssignment
input = input  # @ReservedAssignment

# Import all names from builtins
D = globals()
__original = {}
try:
    D.update(__builtins__)
    __original.update(__builtins__)
except TypeError:
    for value in dir(__builtins__):
        D[value] = getattr(__builtins__, value)
        __original[value] = getattr(__builtins__, value)
        del value

# Recover real name and package
__name__ = __name  # @ReservedAssignment
__package__ = __package  # @ReservedAssignment
del D


#
# Utility functions for messing with builtins
#
def save_name(name, function):
    '''Saves a function to the __builtin__ module'''

    try:
        __builtins__[name] = function
    except TypeError:
        setattr(__builtins__, name, function)


def update(D={}, **kwds):
    '''Update the __builtin__ module with the given values'''

    D = dict(D)
    D.update(kwds)
    for k, v in D.items():
        save_name(k, v)


def restore():
    '''Restores builtins to original state'''

    try:
        D = __builtins__
        D.clear()
        D.update(__original)
    except AttributeError:
        mod = __builtins__
        for name in dir(mod):
            if name in __original:
                value = __original[name]
                setattr(mod, name, value)
            else:
                delattr(mod, name)



import os
import abc
import tempfile
from iospec import types
from iospec.runners import IoObserver
from ejudge import builtins
from ejudge import util
from ejudge.pinteract import Pinteract


__all__ = ['IntegratedLanguage', 'ScriptingLanguage', 'CompiledLanguage']


class LanguageMeta(abc.ABCMeta):
    def __new__(cls, name, bases, namespace):
        new = type.__new__(cls, name, bases, namespace)
        if not namespace.get('abstract', False):
            new.register(**namespace)
        return new

    def register(cls, *, name, description, extensions=[], **kwds):
        """Register class after creation."""

        for ext in extensions:
            cls.registered_extensions[ext] = name
        if extensions:
            cls.registered_language_extensions[name] = extensions[0]
        cls.supported_languages[name] = cls


class LanguageManager(metaclass=LanguageMeta):
    """
    Support for new languages is done by subclassing this class.

    The LanguageManager automatically register new classes using meta
    information that is passed as class attributes. It requires that the
    following attributes should be defined:

    name:
        Short lowercase name for the language. (e.g.: python)
    description:
        A short description of the language (appending version numbers and
        possibly compile flags). (e.g.: Python 3.x)
    extensions:
        A list of valid extensions associated to that language.

    """
    abstract = True
    supported_languages = {}
    registered_extensions = {}
    registered_language_extensions = {}

    class BuildError(Exception):
        pass

    def __init__(self, source):
        self.source = source
        self.context = None

    @classmethod
    def from_language(cls, lang, source):
        try:
            factory = cls.supported_languages[lang]
        except KeyError:
            raise ValueError('invalid language: %r' % lang)
        return factory(source)

    @abc.abstractmethod
    def build_context(self):
        """Returns the build context for the given manager.

        Subclasses should not need to implement an explicit syntax check in this
        function. Override the syntax_check() for this.

        Failed builds can either return None or raise a BuildError."""

        raise NotImplementedError

    @abc.abstractmethod
    def exec(self, inputs, context):
        """Executes the source code in the given context and return the
        resulting test case object

        The context parameter is the result of calling the build function."""

        raise NotImplementedError

    @abc.abstractmethod
    def syntax_check(self):
        raise SyntaxError('invalid syntax')

    def build(self):
        """Builds and check the source code syntax. Also prepares the
        environment before execution.

        This function can be called multiple times and it will return a cached
        build context."""

        if self.context is None:
            self.syntax_check()
            self.context = self.build_context()
            if self.context is None:
                data = util.indent(self.source)
                raise self.BuildError('cannot build source: \n%s' % data)
        return self.context

    def prepare_error(self, ex):
        """Prepares an exception receiving trying to execute
        manager.exec(ctx, inputs) before it is inserted into the error attribute
        of the returning ErrorTestCase."""

        return str(ex)

    def flush_io(self):
        """Flush all io buffers and return what has been collected so far.

        The resulting value is a list of plain input or output nodes."""

        return []

    def run(self, inputs, *, timeout=None, context=None):
        """Executes the program with the given inputs"""

        if context is None:
            context = self.build()

        try:
            func = self.exec
            return remove_trailing_newline(
                    util.timeout(func, args=(inputs, context), timeout=timeout))

        except TimeoutError as ex:
            msg = 'Maximum execution time exceeded: %s sec' % timeout
            return types.ErrorTestCase(
                type='error-timeout',
                data=self.flush_io(),
                timeout=timeout,
                error=msg)

        except RuntimeError as ex:
            data = self.flush_io()
            return remove_trailing_newline(types.ErrorTestCase(
                type='error-exception',
                error=self.prepare_error(ex),
                data=self.flush_io(),
            ))


class IntegratedLanguage(LanguageManager):
    """Base class for all languages that expose hooks for printing and input
    directly to Python's interpreter itself.

    This runner is the most integrated with the ejudge system. Ideally all
    languages should be implemented as subclasses of IntegratedLanguage.
    This may not be feasible or practical for most programming languages,
    though."""

    abstract = True

    def __init__(self, source, globals={}, locals={}, builtins={}):
        super().__init__(source)
        self.observer = IoObserver()
        self.globals = globals.copy()
        self.locals = locals.copy()
        self.builtins = {
            'print': self.observer.print,
            'input': self.observer.input,
        }
        self.builtins.update(builtins)

    def build_context(self):
        self.syntax_check()

        return types.Node(
            globals=self.globals.copy(),
            locals=self.locals.copy(),
        )

    def run(self, inputs, **kwds):
        builtins.update(self.builtins)
        self.observer.flush()
        self.observer.extend_inputs(inputs)

        try:
            result = super().run(inputs, **kwds)
            if result is None:
                result = types.IoTestCase(type='io-plain',
                                          data=self.observer.flush())
        finally:
            builtins.restore()

        return remove_trailing_newline(result)

    def syntax_check(self):
        raise NotImplementedError


class ScriptingLanguage(LanguageManager):
    """Basic support for scripting language."""

    abstract = True

    @property
    def extension(self):
        return self.extensions[0]

    @property
    def shellargs(self):
        raise RuntimeError('shellargs must be overriden in the subclass')

    def build_context(self):
        """Base buildfunc for source code that can be executed as scripts.

        Concrete implementations must provide a default extension for the script
        files and a syntax_check(src) function, that takes a string of code and
        raise BuildProblem if the syntax is invalid."""

        tmpdir = tempfile.mkdtemp()
        tmppath = os.path.join(tmpdir, 'main.' + self.extension)
        with open(tmppath, 'w') as F:
            F.write(self.source)
        return types.Node(
            tempdir=tmpdir,
            temppath=tmppath,
            shellargs=self.shellargs,
        )

    def exec(self, inputs, context):
        return self.exec_pinteract(inputs, context)

    def exec_pinteract(self, inputs, context):
        """Run script as a subprocess and gather results of execution.

        This is a generic implementation. Specific languages or runtimes are
        supported by fixing the shellargs argument of this function. This function
        uses the PInteract() object for communication with scripts.
        """

        def append_non_empty_out(L):
            data = process.receive()
            if data:
                stream.append(types.Node(type='output', data=data))

        tmpdir = context.tempdir
        currpath = os.getcwd()
        shellargs = self.shellargs
        inputs = list(inputs)
        stream = []
        result = types.Node(type='io', data=stream)

        # Execute script in the tmpdir and than go back once execution has
        # finished
        currpath = os.getcwd()
        os.chdir(tmpdir)
        try:
            process = Pinteract(shellargs)

            # Fetch all In/Out strings
            append_non_empty_out(stream)
            while inputs:
                inpt = inputs.pop(0)
                try:
                    process.send(inpt)
                    stream.append(types.Node(type='input', data=inpt))
                except RuntimeError:
                    inputs.append(inpt)
                    return types.ErrorTestCase(
                            type='error-exception', data=stream)
                append_non_empty_out(stream)
        finally:
            os.chdir(currpath)

        # Finish process
        error_ = process.finish()
        assert not any(error_), error_
        return result


    def exec_popen(self, inputs, context):
        """Run script as a subprocess and gather results of execution.

        This is a generic implementation. Specific languages or runtimes are
        supported by fixing the shellargs argument of this function. This function
        uses a Popen() object to interact with the scripts. It should be avoided
        in favor of the PInteract() version.
        """

        tmpdir, _src = ctx

        # Prepare list of inputs and interactions
        inputs = ''.join(str(x) + '\n' for x in template.inputs())
        template = as_atom(template)
        timeout = None if timeout == float('inf') else timeout

        # Execute script in the tmpdir and than go back once execution has finished
        currpath = os.getcwd()
        os.chdir(tmpdir)
        try:
            PIPE = subprocess.PIPE
            STDOUT = subprocess.STDOUT
            process = subprocess.Popen(shellargs, universal_newlines=True,
                                       stdin=PIPE, stdout=PIPE, stderr=STDOUT)
            out, _error = process.communicate(inputs, timeout=timeout)
        except subprocess.CalledProcessError as ex:
            out = IOAtom([], error=ex)
            out.error = error.RuntimeProblem(template=template,
                                                    interaction=out)
            raise
            return out
        except (TimeoutError, subprocess.TimeoutExpired) as ex:
            out = IOAtom([], error=ex)
            out.error = error.TimeExceeded(template=template,
                                                 interaction=out)
            return out

        finally:
            os.chdir(currpath)

        # Try to guess interaction from inputs and outputs
        # For now, it only attempts to do an exact match.
        queue = list(template)[::-1]
        interaction = []
        while queue:
            elem = queue.pop()
            if isinstance(elem, In):
                interaction.append(elem)
            elif out.startswith(str(elem)):
                out = out[len(out):]
                interaction.append(elem)
            else:
                interaction.extend(x for x in reversed(queue) if isinstance(x, In))
                interaction.append(Out(out))

        return IOAtom(interaction)






class CompiledLanguage(LanguageManager):
    abstract = True


    def compiled_builder(ext, shellargs, src):
        """Base buildfunc for source code that must be compiled as part of the
        build process.

        Pass the list of shell arguments to run a command that takes a "main.ext"
        file and compiles it to "main.exe". Raises a BuildProblem if compilation
        return with a non-zero value."""

        # Save source file in temporary dir
        tmpdir = tempfile.mkdtemp()
        tmppath = os.path.join(tmpdir, 'main' + ext)
        with open(tmppath, 'w') as F:
            F.write(src)

        # Compile and return
        currpath = os.getcwd()
        os.chdir(tmpdir)
        try:
            errmsgs = 'compilation is taking too long'
            errmsgs = subprocess.check_output(shellargs, timeout=10)
        except (subprocess.CalledProcessError, TimeoutError):
            raise error.BuildProblem(errmsgs)
        finally:
            os.chdir(currpath)

        return (tmpdir, src)


    def compiled_runner(ctx, template, *, timeout=None, pinteract=True, **kwds):
        """Base runner of compiled programs. Can be used as-is."""

        path = os.path.join(ctx[0], 'main.exe')
        if pinteract:
            return script_runner_pinteract([path], ctx, template, timeout=timeout, **kwds)
        else:
            warnings.warn('pinteract=False is not recommended. There are fundamental unresolved issues with the popen interface')
            return script_runner_popen([path], ctx, template, timeout=timeout, **kwds)


def remove_trailing_newline(case):
    """Remove a trailing newline from the last output node."""

    if case is None:
        return None

    if case.data:
        last = case.data[-1]
        if last.type == 'output' and last.data.endswith('\n'):
            last.data = last.data[:-1]
    return case


if __name__ == '__main__':
    src = 'print("hello world!")'
    pymgm = PythonManager(src)
    out = pymgm.run([])
    print(out)


import math
import os
import pytest
from ejudge.sandbox import run


def write_to_file(path, data):
    with open(path, 'w') as F:
        F.write(data)


def test_basic_sandbox_run():
    assert run(math.sqrt, args=(4.0,)) == 2.0


def test_forbidden_sandbox_run():
    try:
        write_to_file('trash.dat', 'foo')
        with pytest.raises(RuntimeError):
            run(os.unlink, args=('trash.dat',))
    finally:
        os.unlink('trash.dat')


if __name__ == '__main__':
    pytest.main('test_sandbox.py')



import pytest
from ejudge.pinteract import Pinteract


def test_simple_interaction():
    p = Pinteract(['echo', 'foo'])
    assert p.receive() == 'foo\n'


def test_io_interaction():
    # Simple echo program
    p = Pinteract(['python3', '-c', 'print(input("foo: "))'])
    assert p.receive() == 'foo: '
    p.send('bar')
    assert p.receive() == 'bar\n'

if __name__ == '__main__':
    pytest.main('test_pinteract.py')


template = '''
# This files demonstrates the IO templates syntax.
# Text following a "#" are comments

# This is the first example
# The syntax for input prompts is: "prompt message": "input"
# The outputs are preceeded by an "-->" arrow.
"x: ": "a"
"y: ": "b"
--> "result: ab"

# Different examples are separated by two newlines
# The quotes in the interaction strings are largely unecessary, unless
# one needs a very precise control of whitespace
"x: ": b
"y: ": c
--> "result: bc"

# Since the colon character ":" may appear so frequently in input
# prompts, we also accept the double collon "::" syntax
x:: foo
y:: bar
--> "result: foobar"

# Finally, interaction can be specified in a single line for compactness
x:: 1; y:: 2 --> "result: 3"

# Remember to separate each test by at least two newlines!
x:: 1.0; y:: 2.0 --> "result: 3.0"

x:: 1; y:: 5 --> "result: 6"

x:: 1; y:: 5.0 --> "result: 6.0"

x:: 2.0; y:: 5.0 --> "result: 7.0" # we can put comments here too!
'''

if __name__ == '__main__':
    with open('simple.io', 'w') as F:
        F.write(template)



import pytest
from ejudge import io
from iospec import parse_string, types


@pytest.fixture
def iospec():
    return parse_string(
        'name: <foo>\n'
        'hello foo!\n'
        '\n'
        'name: <bar>\n'
        'hello bar!'
    )


@pytest.fixture
def src_ok():
    return (
        'name = input("name: ")\n'
        'print("hello %s!" % name)'
    )


@pytest.fixture
def src_bad():
    return (
        'name = input("name: ")\n'
        'print("hello %s." % name)'
    )


def test_simple_case_success_run(src_ok):
    cases = io.run(src_ok, ['foo'], lang='python', sandbox=False).cases
    data = cases[0].data
    assert cases
    assert len(cases) == 1
    assert data[0].data == 'name: '
    assert data[1].data == 'foo'
    assert data[2].data == 'hello foo!'


def test_simple_case_success_timeout(src_ok):
    cases = io.run(src_ok, ['foo'], lang='python', timeout=0.5, sandbox=False).cases
    data = cases[0].data
    assert len(cases) == 1
    assert data[0].data == 'name: '
    assert data[1].data == 'foo'
    assert data[2].data == 'hello foo!'


def test_simple_case_success_sandbox(src_ok):
    cases = io.run(src_ok, ['foo'], lang='python', sandbox=True).cases
    data = cases[0].data
    assert len(cases) == 1
    assert data[0].data == 'name: '
    assert data[1].data == 'foo'
    assert data[2].data == 'hello foo!'


def test_grading_correct_answer(iospec, src_ok):
    feedback = io.grade(src_ok, iospec, lang='python', sandbox=False)
    assert isinstance(feedback.answer_key, types.TestCaseNode)
    assert isinstance(feedback.case, types.TestCaseNode)
    assert feedback.grade == 1
    assert feedback.message is None
    assert feedback.status == 'ok'


def test_grading_incorrect_answer(iospec, src_bad):
    feedback = io.grade(src_bad, iospec, lang='python', sandbox=False)
    assert feedback.grade == 0
    assert feedback.status == 'wrong-answer'
    assert feedback.title == 'Wrong Answer'


def test_python_script_run(src_ok):
    cases = io.run(src_ok, ['foo'], lang='python3', sandbox=False).cases
    data = cases[0].data
    assert len(cases) == 1
    assert data[0].data == 'name: '
    assert data[1].data == 'foo'
    assert data[2].data == 'hello foo!'


if __name__ == '__main__':
    pytest.main('test_io_grader_python.py')



import io
import traceback
from ejudge.langs import IntegratedLanguage, ScriptingLanguage


class Python3Mixin:
    def syntax_check(self):
        try:
            compile(self.source, '<string>', 'exec')
        except SyntaxError:
            out = io.StringIO()
            traceback.print_exc(file=out, limit=0)
            msg = out.getvalue()
            raise SyntaxError(msg)

    def prepare_error(self, ex):
        """Formats exception"""

        exname = type(ex).__name__
        messages = []
        codelines = self.source.splitlines()
        tb = ex.__traceback__

        tblist = reversed(traceback.extract_tb(tb))
        for (filename, lineno, funcname, text) in tblist:
            if 'judge' in filename:
                break
            if filename == '<string>':
                text = codelines[lineno - 1].strip()
            messages.append((filename, lineno, funcname, text))

        messages.reverse()
        messages = traceback.format_list(messages)
        messages.insert(0, 'Traceback (most recent call last)')
        messages.append('%s: %s' % (exname, ex))
        return '\n'.join(messages)


class PythonManager(Python3Mixin, IntegratedLanguage):
    name = 'python'
    description = 'Python 3.x'
    extensions = ['py', 'py3']

    def exec(self, inputs, context):
        assert context is not None
        code = compile(self.source, '<string>', 'exec')
        exec(code, context.globals, context.locals)


class PythonScriptManager(Python3Mixin, ScriptingLanguage):
    name = 'python3'
    description = 'Python 3.x'
    extension = 'py'
    shellargs = ['python3', 'main.py']


class Python2Manager(ScriptingLanguage):
    name = 'python2'
    description = 'Python 2.7'
    extension = 'py'
    shellargs = ['python2', 'main.py']

    def syntax_check(self):
        pass


def python_io_builder(src):
    """Default buildfunc for python code.

    Checks for syntax errors."""

    try:
        compile(src, '<string>', 'exec')
    except SyntaxError:
        out = io.StringIO()
        traceback.print_exc(file=out, limit=0)
        msg = out.getvalue()
        raise error.BuildProblem(code=src, msg=msg)
    return src



def _format_exception(ex, src):
    """Formats exception"""

    exname = type(ex).__name__
    messages = []
    codelines = src.splitlines()
    tb = ex.__traceback__

    tblist = reversed(traceback.extract_tb(tb))
    for (filename, lineno, funcname, text) in tblist:
        if 'judge' in filename:
            break
        if filename == '<string>':
            text = codelines[lineno - 1].strip()
        messages.append((filename, lineno, funcname, text))

    messages.reverse()
    messages = traceback.format_list(messages)
    messages.insert(0, 'Traceback (most recent call last)')
    messages.append('%s: %s' % (exname, ex))
    return '\n'.join(messages)



"""
#python3_builder = partial(script_builder, '.py', python3_syntax)
python3_runner = partial(script_runner, ['python3', 'main.py'])

#
# Register all python io graders
#
ioregister('python', 'Python 3.x', python_io_builder, python_io_runner,
           ['py', 'py3'])

# Python 2
ioregister('python2', 'Python 2.x',
           partial(script_builder, '.py', python2_syntax),
           partial(script_runner, ['python2', 'main.py']),
           ['py2'])

# Python 3
ioregister('python3', 'Python 3.x',
           partial(script_builder, '.py', python3_syntax),
           partial(script_runner, ['python3', 'main.py']),
           [])


# Pytuga
ioregister('pytuga', 'Pytuguês',
           partial(script_builder, '.pytg', pytuga_syntax),
           partial(script_runner, ['pytuga', 'main.pytg']),
           ['.pytg'])
"""


import unittest
from judge import error
from judge.graders import iograder


#
# Code fragments
#
templ1 = r'''
1; 2; --> "result: 3"  

-1; 2; --> "result: 1"
'''

src1_ok = r'''
#include <stdio.h>

void main() {
    int x=0, y=0;
    scanf("%i", &x);
    scanf("%i", &y);
    printf("result: %i\n", x + y);
}
''' 

src1_presentation = r'''
#include <stdio.h>

void main() {
    int x=0, y=0;
    scanf("%i", &x);
    scanf("%i", &y);
    printf("Result:%i\n", x + y);
}
''' 

src1_wrong = r'''
#include <stdio.h>

void main() {
    int x=0, y=0;
    scanf("%i", &x);
    scanf("%i", &y);
    printf("result: %i\n", x + y + 1);
}
'''

src1_missing = r'''
#include <stdio.h>

void main() {
    int x=0, y=0;
    scanf("%i", &x);
    printf("result: %i\n", x + y);
}

'''

src1_runtime = r'''
#include <stdio.h>

void main() {
    int x=0, y=0;
    scanf("%i", &x);
    scanf("%i", &y);
    printf("result: %i\n");
}
'''

src1_extra = r'''
#include <stdio.h>

void main() {
    int x=0, y=0, z=1;
    scanf("%i", &x);
    scanf("%i", &y);
    scanf("%i", &z);
    printf("result: %i\n", x + y + z);
}
'''

src1_timeout = r'''
#include <unistd.h>

void main() {
    sleep(10000);
}
'''


class IOCGraderTestCase(unittest.TestCase):
    sandbox = False
    lang = 'tcc'
    
    def test_src1_ok(self):
        grade = iograder(src1_ok, templ1, sandbox=self.sandbox, lang=self.lang)
        self.assertEqual(grade.value, 1)
        self.assertEqual(grade.feedback, None)

    def test_src1_wrong(self):
        grade = iograder(src1_wrong, templ1, sandbox=self.sandbox, lang=self.lang)
        self.assertEqual(grade.value, 0)
        self.assertIsInstance(grade.feedback.error, error.WrongAnswer)

    def test_src1_presentation(self):
        grade = iograder(src1_presentation, templ1, sandbox=self.sandbox, lang=self.lang)
        self.assertEqual(grade.value, 0)
        self.assertIsInstance(grade.feedback.error, error.PresentationError)
        
    def test_src1_missing(self):
        grade = iograder(src1_missing, templ1, sandbox=self.sandbox, lang=self.lang)
        self.assertEqual(grade.value, 0)
        self.assertIsInstance(grade.feedback.error, error.EarlyTermination)

    def test_src1_runtime(self):
        grade = iograder(src1_runtime, templ1, sandbox=self.sandbox, lang=self.lang)
        self.assertEqual(grade.value, 0)
        #self.assertIsInstance(grade.feedback.error, error.RuntimeConditionError)
        self.assertIsInstance(grade.feedback.error, error.WrongAnswer)
        
    def test_src1_extra(self):
        grade = iograder(src1_extra, templ1, sandbox=self.sandbox, lang=self.lang)
        self.assertEqual(grade.value, 0)
        self.assertIsInstance(grade.feedback.error, error.TimeExceeded)
        
    def test_src1_timeout(self):
        grade = iograder(src1_timeout, templ1, timeout=0.05, sandbox=self.sandbox, lang=self.lang)
        self.assertEqual(grade.value, 0)
        self.assertIsInstance(grade.feedback.error, error.TimeExceeded)


    
if __name__ == '__main__':
    unittest.main()


import io
import os
import unittest
import functools
from judge.bin.script import gradefile
from judge.tests.examples.simple_template import template as simple_template


class TestSimpleExamples(unittest.TestCase):

    def test_simple(self):
        path = os.path.join(basepath, 'simple.py')
        with open(path) as F:
            grade = gradefile(F, io.StringIO(simple_template), '.py')
        assert grade.message is None, grade
        assert grade.value == 1, grade


basepath = os.path.split(__file__)[0]
for suffix in ['build', 'extrainput', 'presentation', 'timeout', 'unexpected',
               'unusedinput', 'wrong']:

    with open(os.path.join(basepath, 'simple_%s.py' % suffix)) as F:
        src = F.read()

    with open(os.path.join(basepath, 'simple_%s.txt' % suffix)) as F:
        response = F.read()
        response = '\n'.join(response.splitlines()[:-1])

    def method(src, response):
        src = io.StringIO(src)
        template = io.StringIO(simple_template)
        grade = gradefile(src, template, '.py', timeout=0.5)
        msg = '\nGot:\n\n%s\n\nExpect:\n %s' % (grade.message, response)
        assert grade.message == response, msg
        assert grade.value == 0, grade

    method = functools.partial(method, src, response)
    method_name = 'test_%s' % suffix
    setattr(TestSimpleExamples, method_name, method)


if __name__ == '__main__':
    TestSimpleExamples.test_simple(None)
    TestSimpleExamples.test_wrong()



from ejudge.langs import ScriptingLanguage


class Pytuga2Manager(ScriptingLanguage):
    name = 'pytuga'
    description = 'Pytuguês'
    extensions = ['pytg']
    shellargs = ['pytuga', 'main.py']

    def syntax_check(self):
        pass


import time
import psutil
import pexpect
inf = float('inf')


class Pinteract:
    """
    A class for conveniently interacting with a child process.
    """

    def __init__(self, command, timeout=None, encoding='utf8'):
        if timeout == inf:
            timeout = None
        self.__burnt = False
        self._remaining_time = timeout or float('inf')
        self._process = pexpect.spawn(command[0], command[1:])
        self._process.setecho(False)
        self._psdata = psutil.Process(self._process.pid)
        self.encoding = encoding
        self.timeout = timeout
        self.pid = self._process.pid
        
    def burn(self, duration):
        """Burn at most the given time (in seconds) waiting for the process to
        leave the "running" status."""
        
        t0 = time.time()
        duration = min(duration, self._remaining_time)
        duration = max(duration, 0)
        for _ in range(int(duration * 100)):
            status = self.status()
            if status == 'running':
                time.sleep(0.01)
            elif status in ['sleeping', 'zombie', 'dead']:
                break
            else:
                raise RuntimeError('status: %s' % status)
        self._remaining_time -= time.time() - t0
        
    def __burn_once(self):
        """Burn at most timeout at the first call to receive()"""
        
        if not self.__burnt:
            if self.timeout is None:
                self.burn(1)
            else:
                self.burn(self.timeout)
        
    def status(self):
        """Return a string with the status code for the process"""

        try:
            return self._psdata.status()
        except psutil.NoSuchProcess:
            return 'dead'
    
    def is_running(self):
        """Return True if child process is still running."""
        
        return self.status() == 'running'
                    
    def is_dead(self):
        """Return True if child process is dead."""
        
        return self.status() in ['zombie', 'dead']

    def is_sleeping(self):
        """Return True if child process is sleeping."""
        
        return self.status() == 'sleeping'

    def receive(self):
        """Read the output of the child process."""

        self.__burn_once()
        t0 = time.time()
        data = []
        last = 'non empty'
        while last:
            try:
                last = self._process.read_nonblocking(1, timeout=0.05)
            except pexpect.EOF:
                break
            except pexpect.TIMEOUT:
                if self.status() == 'sleeping':
                    break
                elif self.status() == 'running':
                    tf = time.time()
                    self._remaining_time -= tf - t0
                    t0 = tf
                    if self._remaining_time <= 0:
                        raise TimeoutError
                    break
                else:
                    print('status:', self.status())
                    raise
            data.append(last)

        self._remaining_time -= time.time() - t0        
        data = b''.join(data)
        if self.encoding is not None:
            data = data.decode(self.encoding)
        return data.replace('\r\n', '\n')
    
    def send(self, data, end=None):
        """Write some input string of data to the child process"""
        
        t0 = time.time()
        if self.is_dead():
            raise RuntimeError('trying to send message to a closed process')
        
        if end is None:
            self._process.sendline(data)
        else:
            self._process.send(data + end)
        self._remaining_time -= time.time() - t0

    def finish(self):
        """Finish process execution. Return a tuple with any unread
        (stdout, stderr) messages"""

        out = self.receive()
        self._process.kill(9)
        return out
    
if __name__ == '__main__':
    sub = Pinteract(['ls', '-lha'])
    print(sub.receive())
    print(sub.finish())
    print('OK', flush=True)
    
    
    sub = Pinteract(['python', 'tests/examples/simple.py'])
    print(sub.receive())
    print(sub.send('1'))
    print(sub.receive())
    print(sub.send('2'))
    print(sub.receive())
    print(sub.finish())
    print('OK', flush=True)



from functools import partial

#
# C/C++ support for IO problems
#
"""
# TCC
ioregister(
    'tcc', 'ANSI C (tcc compiler)',
    partial(compiled_builder, '.c', ['tcc', 'main.c', '-o', 'main.exe', '-w']),
    compiled_runner,
    ['c'],
)

# GCC
ioregister(
    'gcc', 'ANSI C (gcc compiler)',
    partial(compiled_builder, '.c', ['gcc', 'main.c', '-o', 'main.exe']),
    compiled_runner,
    ['c'],
)

# Clang
ioregister(
    'clang', 'ANSI C (clang compiler)',
    partial(compiled_builder, '.c', ['clang', 'main.c', '-o', 'main.exe']),
    compiled_runner,
    ['c'],
)
"""



import os
import decimal
import functools
from iospec.parser import TemplateNode, ErrorTestCase
from ejudge import util
from ejudge.feedback import get_feedback
from ejudge.sandbox import run as run_sandboxed
from ejudge.langs import LanguageManager


# Python print function using the standard stdout
__all__ = ['grade', 'run', 'BuildError']
BuildError = LanguageManager.BuildError
grade = run = None


def prepare_manager(func):
    """Decorate functions in this block to receive a configured manager"""

    @functools.wraps(func)
    def decorated(src, *args, path=None, lang=None, manager=None, raises=False,
                  sandbox=False, timeout=None, **kwds):
        # Fix language
        if manager is None:
            if lang is None:
                ext = os.path.splitext(path or src.name)[1] or '.'
                try:
                    lang = LanguageManager.registered_extensions[ext[1:]]
                except KeyError:
                    raise ValueError('unknown extension: %r' % ext)

            # Normalize source string
            if not isinstance(src, str):
                src = src.read()
            manager = LanguageManager.from_language(lang, src)

        # Build
        try:
            manager.build()
        except BuildError as ex:
            if raises:
                raise
            return build_error_tree(ex)

        # Run task
        args += (manager,)
        if sandbox:
            return run_sandboxed(func, args=args, kwargs=kwds, timeout=timeout)
        elif timeout:
            return util.timeout(func, args=args, kwargs=kwds, timeout=timeout)
        else:
            return func(*args, **kwds)

    globals()[func.__name__[1:]] = decorated
    return func


@prepare_manager
def _grade(iospec, manager, fast=True):
    """
    Grade the string of source code by comparing the results of all inputs and
    outputs in the given template structure.


    Parameters
    ----------

    src : str or file object
        The source string for the code or a file object
    iospec : IOSpec parse tree
        The expected template for correct answers
    lang : str
        Programming language for the given source code. The judge accepts the
        following languages. Users can register plugins to support additional
        languages or to override the default behavior or accepted languages.

        +===========+==========================================================+
        | Value     | Description                                              |
        +===========+==========================================================+
        | python    | For Python 3.x code. Default runner.                     |
        +-----------+----------------------------------------------------------+
        | python2   | Executes in a separate Python 2 interpreter.             |
        +-----------+----------------------------------------------------------+
        | python3   | Executes in a separate Python 3 interpreter. Used mostly |
        |           | for debuging since it lack in features and performance   |
        |           | compared to the default "python" runner.                 |
        +-----------+----------------------------------------------------------+
        | tcc       | Compile C code with the tiny C compiler                  |
        +-----------+----------------------------------------------------------+
        | clang     | Compile C code with clang                                |
        +-----------+----------------------------------------------------------+
        | gcc       | Compile C code with gcc                                  |
        +-----------+----------------------------------------------------------+
        | C         | Uses the first method available: tcc, clang or gcc       |
        +-----------+----------------------------------------------------------+

    sandbox : bool
        If True, code will run in a sandboxed environment as the `nobody` user.
        It is necessary to have your system properly configured in order to do
        this.
    timeout : float
        Maximum time (in seconds) for the complete test to run.

    Specific backends may support additional keyword arguments. The most common
    ones are shown bellow

    namespace (python): dict
        The globals() namespace used to run a python script.


    Returns
    -------

    A named tuple of (grade, feedback) with a decimal grade normalized to 1 and
    None or a feedback structure.
    """

    if not iospec.cases:
        raise ValueError('cannot grade an iospec that has no cases')

    value = decimal.Decimal(1)
    feedback = None

    for answer_key in iospec.cases:
        inputs = answer_key.inputs()
        case = manager.run(inputs)
        curr_feedback = get_feedback(case, answer_key)

        if feedback is None:
            feedback = curr_feedback

        if curr_feedback.grade < value:
            feedback = curr_feedback
            value = curr_feedback.grade

            if value == 0 and fast:
                break

    return feedback


@prepare_manager
def _run(inputs, manager=None):

    """Runs program with the given list of inputs and returns the
    corresponding IoSpecTree.

    Parameters
    ----------

    src : str or file
        The source code for the test program
    inputs : sequence
        A sequence of input strings. If input is a sequence of sequences,
        this function will perform multiple test cases.
    lang : str
        The name for the source code language. See :func:`ejudge.graders.io.grade`
        for more details.
    timeout : float
        A time limit for the entire run (in seconds). If this attribute is not
        given, the program will run without any timeout. This can be potentially
        dangerous if the input program has an infinite loop.
    sandbox : bool
        Controls if code is run in sandboxed mode or not. Sandbox protection
        is the default behavior on supported platforms.
    raises : bool
        If True, raise a BuildError if the build process fails. The default
        behavior is to return a IoSpecTree with a single test case of type
        'error-build'.
    path : str
        The absolute file path for the input string or file object.

    Returns
    -------

    A :cls:`iospec.IoSpecTree structure. If inputs is a sequence of strings,
    the resulting tree will have a single test case in its "cases" attribute.
    """

    inputs = util.pushbackiter(inputs)
    first = next(inputs)
    inputs.push(first)

    if isinstance(first, str):
        cases = [manager.run(inputs)]
    else:
        cases = [manager.run(values) for values in inputs]
    return TemplateNode(cases=cases)


def build_error_tree(msg):
    return TemplateNode(cases=[ErrorTestCase(type='error-build', message=msg)])


if __name__ == '__main__':
    from iospec import parse
    from pprint import pprint


    template = parse('../tests/examples/simple.iospec')
    with open('../tests/examples/simple_wrong.py') as F:
        g = grade(F, template, timeout=None, sandbox=False)
    pprint(g)



import os
import pwd
import pickle
import base64


# Sets the UID to nobody in order to prevent harm...
userinfo = pwd.getpwnam('nobody')
os.setuid(userinfo.pw_uid)

# Start interaction with Python master: transfer data using b85 encoded pickled
# streams. The main process reads an input string from stdin and prints back
# the result to stdout. This is done until an empty string is received.
while True:
    data = input()
    if not data:
        break
    data = base64.b85decode(data)
    func, args, kwds = pickle.loads(data)
    result = func(*args, **kwds)
    out = base64.b85encode(pickle.dumps(result))
    print(out.decode('utf8'))


"""
Uses Linux kernel capabilities to make sandbox possible. It runs all scripts
as the `nobody` user. For this to work we need a python executable with the
SETUID capability enable. Do these steps::

    $ cp /usr/bin/python3.4 /usr/bin/python_setuid
    $ setcap cap_setuid+ep /usr/bin/python_setuid

Of course the first command should be adapted to the correct name/version of
your Python executable. Remember that in most distributions /usr/bin/python or
/usr/bin/python3 are simply symlinks to the correct Python executable. `setcap`
do not work in symlinks, so you should find the correct Python executable.
DO NOT enable SETUID capability in the main Python interpreter because this can
make your system vulnerable. This library would not use it anyway...
"""


def run(target, args=(), kwargs=None, timeout=None):
    """Run target function in a sandboxed environment and return the results.

    Everything is pickle-encoded and transmitted to the sandboxed children
    process. All arguments must be pickable and any modification the target
    function makes to the input arguments is not transmitted back from the
    function call.
    """
    
    import pickle
    import base64
    from subprocess import PIPE, Popen

    # Encode message
    kwargs = dict(kwargs or {})
    args = tuple(args)
    message = pickle.dumps((target, args, kwargs))
    message = base64.b85encode(message) + b'\n\n'
    
    # Transmit message to subprocess
    cmd = ['python_setuid', '-m', 'ejudge.sandbox']
    proc = Popen(cmd, stdin=PIPE, stdout=PIPE, stderr=PIPE)
    response, err = proc.communicate(input=message, timeout=timeout)
    if proc.poll() != 0:
        raise RuntimeError(err.decode('utf8'))
    
    response = base64.b85decode(response[:-1])
    return pickle.loads(response)



import io
import sys
import signal
import functools
import html
from threading import Thread


#
# String formmating
#
def indent(st, indent=4):
    '''Indent string by the given number of spaces or the given indentation
    string'''

    indent = ' ' * indent if isinstance(indent, int) else indent
    lines = [indent + x for x in st.splitlines()]
    return '\n'.join(lines)


def html_escape(x, keep_newlines=False):
    '''Escape unsafe HTML characters such as < > & etc'''

    if keep_newlines:
        data = [html_escape(x) for x in x.splitlines()]
        return '\n'.join(data)
    return html.escape(x)


#
# Special IO functions for python script interactions
#
_print_func = print


def capture_print(*args, **kwds):
    '''A print-like function that return the formatted string instead of
    printing it on screen'''

    out, err = sys.stdout, sys.stderr
    out_io = sys.stdout = sys.stderr = io.StringIO()
    try:
        _print_func(*args, **kwds)
    finally:
        sys.stdout, sys.stderr = out, err
    return out_io.getvalue()


#
# Execution with time limits
#
def __timeout_handler(signum, frame):
    '''Helper function for the timeout() function.'''

    raise TimeoutError()


def timeout(func, args=(), kwargs={}, timeout=1.0, threading=True,
            raises=True):
    '''Execute callable `func` with timeout. If timeout is None or zero, 
    ignores any timeout exceptions.

    If timeout exceeds, raises a TimeoutError'''

    if not timeout or not 1 / timeout:
        return func(*args, **kwargs)

    if threading:
        result = []
        exceptions = []

        def target():
            try:
                result.append(func(*args, **kwargs))
            except Exception as ex:
                exceptions.append(ex)

        thread = Thread(target=target)
        thread.start()
        thread.join(timeout)
        if thread.is_alive():
            if raises:
                raise TimeoutError
            else:
                return None
        else:
            try:
                return result.pop()
            except IndexError:
                raise exceptions.pop()
    else:
        signal.signal(signal.SIGALRM, __timeout_handler)
        signal.alarm(timeout)
        try:
            result = func(*args, **kwargs)
        except TimeoutError:
            if raises:
                raise
            else:
                return None
        finally:
            signal.alarm(0)

        return result


#
# Lazy evaluation
#
class lazy(object):
    '''Lazy accessor'''

    def __init__(self, func):
        self.func = func

    def __get__(self, obj, cls=None):
        if obj is None:
            return self
        value = self.func(obj)
        setattr(obj, self.func.__name__, value)
        return value


class pushbackiter:
    """A simple push back iterator: push()'d items are restored to iteration.

    Example
    -------

    >>> it = pushbackiter([1, 2, 3]);
    >>> it.push(0); list(it)
    [0, 1, 2, 3]
    """

    def __init__(self, it):
        self.iter = iter(it)
        self.buffer = []

    def __iter__(self):
        return self

    def __next__(self):
        buffer = self.buffer
        while buffer:
            return buffer.pop()
        return next(self.iter)

    def push(self, value):
        """Send a value to the begining of iteration."""

        self.buffer.append(value)


if __name__ == '__main__':
    from doctest import testmod
    testmod()


import unittest
from judge import error
from judge.parsers import IOGroup, In, Out
from judge.graders import iograder, iorun, iorunonce


#
# Code fragments
#
templ1 = r'''
1; 2; --> "result: 3\n" 

-1; 1; --> "result: 0\n"
'''

src1_ok = '''
x = int(input())
y = int(input())
print('result:', x + y)
'''

src1_presentation = '''
x = int(input())
y = int(input())
print('Result: ', x + y)
''' 

src1_presentation_msg = '''
------------------
Presentation Error
------------------

Expected answer:
    [ 1 ]
    [ 2 ]
    result: 3

Obtained:
    [ 1 ]
    [ 2 ]
    Result:  3
''' 

src1_wrong = '''
x = int(input())
y = int(input())
print('result:', x + y + 1)
'''

src1_wrong_msg = '''
------------
Wrong Answer
------------

Expected answer:
    [ 1 ]
    [ 2 ]
    result: 3

Obtained:
    [ 1 ]
    [ 2 ]
    result: 4
''' 

src1_missing = '''
x = int(input())
print('result:', x + 2)
'''

src1_missing_msg = '''
-----------------
Early Termination
-----------------

Expected answer:
    [ 1 ]
    [ 2 ]
    result: 3

Obtained:
    [ 1 ]
    result: 3

Hint:
    Your program did not used all expected inputs.
''' 

src1_runtime = '''
x = int(input())
y = int(input())
print('result:', x + y + z)
'''

src1_runtime_msg = '''
---------------
Runtime Problem
---------------

Expected answer:
    [ 1 ]
    [ 2 ]
    result: 3

Obtained:
    [ 1 ]
    [ 2 ]

Error message:
    Traceback (most recent call last)
      File "<string>", line 4, in <module>
        print('result:', x + y + z)
    
    NameError: name 'z' is not defined

''' 

src1_extra = '''
x = int(input())
y = int(input())
z = int(input())
print('result:', x + y + z)
'''

src1_extra_msg = '''
-------------
Time Exceeded
-------------

Expected answer:
    [ 1 ]
    [ 2 ]
    result: 3

Obtained:
    [ 1 ]
    [ 2 ]

Hint:
    Your program took more than 10.00 seconds to run.
''' 

src1_timeout = '''
import time
time.sleep(1)
'''

src1_timeout_msg = '''
-------------
Time Exceeded
-------------

Expected answer:
    [ 1 ]
    [ 2 ]
    result: 3

Hint:
    Your program took more than 0.05 seconds to run.
'''


class IOPyGraderTestCase(unittest.TestCase):
    sandbox = False
    
    def test_src1_ok(self):
        grade = iograder(src1_ok, templ1, sandbox=self.sandbox)
        self.assertEqual(grade.value, 1)
        self.assertEqual(grade.feedback, None, grade.feedback)
        
    def test_src1_wrong(self):
        grade = iograder(src1_wrong, templ1, sandbox=self.sandbox)
        self.assertEqual(grade.value, 0)
        self.assertIsInstance(grade.feedback.error, error.WrongAnswer)

    def test_src1_wrong_message(self):
        grade = iograder(src1_wrong, templ1, sandbox=self.sandbox)
        out = str(grade.feedback)
        self.assertEqual(out.strip(), src1_wrong_msg.strip())

    def test_src1_presentation(self):
        grade = iograder(src1_presentation, templ1, sandbox=self.sandbox)
        self.assertEqual(grade.value, 0)
        self.assertIsInstance(grade.feedback.error, error.PresentationError)
        
    def test_src1_presentation_message(self):
        grade = iograder(src1_presentation, templ1, sandbox=self.sandbox)
        out = str(grade.feedback)
        self.assertEqual(out.strip(), src1_presentation_msg.strip())
        
    def test_src1_missing(self):
        grade = iograder(src1_missing, templ1, sandbox=self.sandbox)
        self.assertEqual(grade.value, 0)
        self.assertIsInstance(grade.feedback.error, error.EarlyTermination)
        
    def test_src1_missing_message(self):
        grade = iograder(src1_missing, templ1, sandbox=self.sandbox)
        out = str(grade.feedback)
        self.assertEqual(out.strip(), src1_missing_msg.strip())

    def test_src1_runtime(self):
        grade = iograder(src1_runtime, templ1, sandbox=self.sandbox)
        self.assertEqual(grade.value, 0)
        self.assertIsInstance(grade.feedback.error, error.RuntimeProblem)

    def test_src1_runtime_message(self):
        grade = iograder(src1_runtime, templ1, sandbox=self.sandbox)
        out = str(grade.feedback)
        self.assertEqual(out.strip(), src1_runtime_msg.strip())

    def test_src1_extra(self):
        grade = iograder(src1_extra, templ1, sandbox=self.sandbox)
        self.assertEqual(grade.value, 0)
        self.assertIsInstance(grade.feedback.error, error.TimeExceeded)
    
    def test_src1_extra_message(self):
        grade = iograder(src1_extra, templ1, sandbox=self.sandbox)
        out = str(grade.feedback)
        self.assertEqual(out.strip(), src1_extra_msg.strip())
        
    def test_src1_timeout(self):
        grade = iograder(src1_timeout, templ1, timeout=0.05, sandbox=self.sandbox)
        self.assertEqual(grade.value, 0)
        self.assertIsInstance(grade.feedback.error, error.TimeExceeded)

    def test_src1_timeout_message(self):
        grade = iograder(src1_timeout, templ1, timeout=0.05, sandbox=self.sandbox)
        out = str(grade.feedback)
        self.assertEqual(out.strip(), src1_timeout_msg.strip())

    
class IORunTestCase(unittest.TestCase):
    def test_iorunonce_ok(self):
        templ = IOGroup.fromstring(templ1)
        for atom in templ:
            interaction = iorunonce(src1_ok, atom)
            self.assertEqual(list(atom), list(interaction))
        
    def test_iorunonce_wrong(self):
        interaction = list(iorunonce(src1_wrong, (1, 5)))
        expected = [In('1'), In('5'), Out('result: 7\n')] 
        self.assertEqual(interaction, expected)
        
    def test_iorun_ok(self):
        templ = IOGroup.fromstring(templ1)
        self.assertEqual(iorun(src1_ok, templ), templ)
        
    def test_iorun_from_inputs_ok(self):
        templ = IOGroup.fromstring(templ1)
        inputs = [['1', '2'], ['-1', '1']]
        self.assertEqual(iorun(src1_ok, inputs), templ)


class IOResultCanRenderHtmlTestCase(unittest.TestCase):
    def test_wrong(self):
        iograder(src1_wrong, templ1, format='html')
        
    def test_timeout(self):
        iograder(src1_timeout, templ1, format='html')
        
    def test_presentation(self):
        iograder(src1_presentation, templ1, format='html')

    def test_missing(self):
        iograder(src1_missing, templ1, format='html')
        
    def test_runtime(self):
        iograder(src1_runtime, templ1, format='html')
        
    def test_extra(self):
        iograder(src1_extra, templ1, format='html')


if __name__ == '__main__':
    unittest.main()
    


import decimal
from iospec.types import Node
from ejudge.util import html_escape, indent


ERROR_TITLES = {
    'error-timeout': 'Timeout Error',
}


def get_feedback(case, answer_key):
    """Return a feedback structure that represents the expected error"""

    if case == answer_key:
        return Feedback(case, answer_key)

    feedback = Feedback(case, answer_key, grade=decimal.Decimal(0))

    # Presentation errors
    if presentation_equal(case, answer_key):
        feedback.title = 'Presentation Error'
        feedback.status = 'presentation-error'
        feedback.grade = decimal.Decimal(0.5)

    # Wrong answer
    elif case.type.startswith('io'):
        feedback.status = 'wrong-answer'
        feedback.title = 'Wrong Answer'

    # Error messages
    elif case.type.startswith('error'):
        feedback.status = case.type
        try:
            feedback.title = ERROR_TITLES[case.type]
        except KeyError:
            title = ' '.join(reversed(case.title.split('-')))
            feedback.title = title.title()

    else:
        raise ValueError('invalid testcase type: %r' % case.type)

    return feedback


def presentation_equal(case1, case2):
    """Return True if both cases are equal after casefolding and stripping all
    spaces"""

    #TODO: implement
    return case1 == case2


#
# Error message classes
#
class Feedback:
    """User feedback.

    Parameters
    ----------

    value : TestCase
        pass
    title: str
        Title of the message (e.g., Wrong answer)
    """
    
    def __init__(self, case, answer_key,
                 grade=decimal.Decimal(1), status='ok',
                 title=None, message=None, hint=None):

        self.case = case
        self.answer_key = answer_key
        self.grade = grade
        self.status = status
        self.title = title
        self.hint = hint
        self.message = message
        if message is None and case.type.startswith('error'):
            self.message = case.error

    def render_testcase(self, default='', *, format=None):
        """Render the interaction attribute with the given format.

        An optional default can be given if no response exists."""

        format = format or self.format
        if self.interaction is None:
            return default
        else:
            return self.interaction.format(format)

    def render_template(self, default='', format=None):
        """Render the template attribute with the given format

        An optional default can be given if no template exists."""

        format = format or self.format
        if self.template is None:
            return default
        else:
            return self.template.format(format)

    def name(self):
        """A string with the error name"""

        return type(self.error).__name__


    def title_text(self, format='plain'):  # @ReservedAssignment
        """Return a formatted text representing the title"""

        if format == 'plain':
            title = self.title()
            line = '-' * len(title)
            return '\n'.join([line, title, line])
        elif format == 'html':
            title = self.title()
            return '<strong>%s</strong>' % html_escape(title)
        else:
            return self.title_text('plain')

    def render_plain(self, color=None):
        """Generic formatter of plain text messages"""

        title = self.title_text()
        method = 'color' if color else 'plain'
        template = indent(self.render_template(method) or '')
        interaction = indent(self.render_interaction(method) or '')
        message = self.message_color() if color else self.message_plain()
        message = indent(message or '')
        feedback_title = self.FEEDBACK_TITLE

        # Get colors
        color = color or disabled
        H = color.HEADER
        B = color.BOLD
        I = color.INPUTVALUE
        E = color.ENDC

        # Formatted strings
        D = locals()
        title = '%(H)s%(title)s%(E)s\n\n' % D
        expected = '%(B)sExpected answer:%(E)s\n%(template)s\n\n' % D
        obtained = '%(B)sObtained:%(E)s\n%(interaction)s\n\n' % D
        hint = '%(B)s%(feedback_title)s:%(E)s\n%(message)s\n\n' % D

        # Remove empty parts
        if not self.template:
            expected = ''
        if not self.interaction:
            obtained = ''
        if not message.strip():
            hint = ''

        return ''.join([title, expected, obtained, hint])

    def render_color(self):
        """Generic formatter of plain text messages"""

        return self.render_plain(color=color)

    def render_html(self):
        """Generic formatter of html messages"""

        title = self.title()
        template = indent(self.render_template('html'))
        interaction = indent(self.render_interaction('html'))
        hint = indent(self.message_html())
        feedback_title = self.FEEDBACK_TITLE
        has_message = bool(self.message_plain())

        # Pieces
        D = locals()
        title = (
        '<div class="pyjudge-error-box">\n'
        '<div class="pyjudge-error-title">%(title)s</div>\n'
        ) % D
        expected = ((
        '<div class="pyjudge-example">\n'
        '  <p class="pyjudge-example-title">Expected answer (inputs in red):</p>\n'
        '  <div class="pyjudge-example-data">\n'
        '%(template)s\n'
        '  </div>\n'
        '</div>\n'
        ) % D) if self.template else ''
        obtained = ((
        '<div class="pyjudge-example">\n'
        '  <p class="pyjudge-example-title">Obtained:</p>\n'
        '  <div class="pyjudge-example-data">\n'
        '%(interaction)s\n'
        '  </div>\n'
        '</div>\n'
        ) % D) if self.interaction else ''
        hint = ((
        '<div class="pyjudge-hint">\n'
        '  <p class="pyjudge-example-title">%(feedback_title)s:</p>\n'
        '  <div class="pyjudge-hint-data">\n'
        '%(hint)s\n'
        '  </div>\n'
        '</div>\n'
        ) % D) if has_message else ''

        return ''.join([title, expected, obtained, hint, '</div>'])

    def message_plain(self):
        return ''

    def message_color(self):
        return self.message_plain()

    def message_html(self):
        plain = self.message_plain()
        return ('<pre>%s</pre>' % plain if plain is not None else None)

    @property
    def data(self):
        try:
            return self._data
        except AttributeError:
            try:
                self._data = self.render_page()
            except Exception as ex:
                raise RuntimeError('exception caught when rendering message: %r' % ex)
            return self._data
        
    @data.setter
    def data(self, value):
        self._data = value

    def render(self, format=None):  # @ReservedAssignment
        """Render object by the specified format.

        If no format is given, use the default format defined in the
        constructor."""
        
        if format is None:
            format = self.format  # @ReservedAssignment
        try:
            render_format = getattr(self, 'render_' + format)
        except AttributeError:
            raise ValueError('unknown format: %r' % format)
        else:
            return render_format()
            
    def render_plain(self):
        """Plain text rendering"""
        
        return self.args[0]
        
    def render_html(self):
        """HTML rendering.

        The default behavior is to put the html-escaped contents of plain text
        rendering inside <pre> </pre> tags.
        """
        
        plain = self.render_plain()
        return '<pre>\n%s\n</pre>' % html_escape(plain)
    
    def render_latex(self):
        """LaTeX rendering.

        The default behavior is to put the contents of plain text rendering into
        a verbatim environment."""
        
        return '\\begin{verbatim}\n%s\n\end{verbatim}' % self.render_plain()

#
# Color support
# See: https://en.wikipedia.org/wiki/ANSI_escape_code
#
class color:
    HEADER = '\033[1m\033[95m'
    OKBLUE = '\033[94m'
    OKGREEN = '\033[92m'
    WARNING = '\033[93m'
    FAIL = '\033[91m'
    ENDC = '\033[0m'
    BOLD = '\033[1m'
    UNDERLINE = '\033[4m'
    INPUTVALUE = BOLD + FAIL

class disabled:
    BOLD = ''
    HEADER = ''
    OKBLUE = ''
    OKGREEN = ''
    WARNING = ''
    FAIL = ''
    ENDC = ''
    BOLD = ''
    UNDERLINE = ''
    INPUTVALUE = ''
